# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/moseley',
                'doc_host': 'https://fligt.github.io',
                'git_url': 'https://github.com/fligt/moseley',
                'lib_path': 'moseley'},
  'syms': { 'moseley.moseley_law': {'moseley.moseley_law.moseley_law': ('moseley-law.html#moseley_law', 'moseley/moseley_law.py')},
            'moseley.mplot': { 'moseley.mplot.XFluo': ('moseley-law.html#xfluo', 'moseley/mplot.py'),
                               'moseley.mplot.XFluo.__init__': ('moseley-law.html#xfluo.__init__', 'moseley/mplot.py'),
                               'moseley.mplot.XFluo.plot': ('moseley-law.html#xfluo.plot', 'moseley/mplot.py'),
                               'moseley.mplot._colorize': ('moseley-law.html#_colorize', 'moseley/mplot.py'),
                               'moseley.mplot._draw_box': ('moseley-law.html#_draw_box', 'moseley/mplot.py'),
                               'moseley.mplot.make_ptable': ('moseley-law.html#make_ptable', 'moseley/mplot.py'),
                               'moseley.mplot.moseley_law': ('moseley-law.html#moseley_law', 'moseley/mplot.py'),
                               'moseley.mplot.moseley_plot': ('moseley-law.html#moseley_plot', 'moseley/mplot.py'),
                               'moseley.mplot.ptable_plot': ('moseley-law.html#ptable_plot', 'moseley/mplot.py')},
            'moseley.peak_pattern_atlas': { 'moseley.peak_pattern_atlas.PeakPatternAtlas': ( 'a-peak-pattern-atlas.html#peakpatternatlas',
                                                                                             'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas.PeakPatternAtlas.__init__': ( 'a-peak-pattern-atlas.html#peakpatternatlas.__init__',
                                                                                                      'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas.PeakPatternAtlas.plot_atlas': ( 'a-peak-pattern-atlas.html#peakpatternatlas.plot_atlas',
                                                                                                        'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas.PeriodicTable': ( 'a-peak-pattern-atlas.html#periodictable',
                                                                                          'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas.PeriodicTable.__init__': ( 'a-peak-pattern-atlas.html#periodictable.__init__',
                                                                                                   'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas.PeriodicTable.plot_table': ( 'a-peak-pattern-atlas.html#periodictable.plot_table',
                                                                                                     'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas._draw_box': ( 'a-peak-pattern-atlas.html#_draw_box',
                                                                                      'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas._fetch_table': ( 'a-peak-pattern-atlas.html#_fetch_table',
                                                                                         'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas._sort_ptrn_list': ( 'a-peak-pattern-atlas.html#_sort_ptrn_list',
                                                                                            'moseley/peak_pattern_atlas.py'),
                                            'moseley.peak_pattern_atlas.colorize': ( 'a-peak-pattern-atlas.html#colorize',
                                                                                     'moseley/peak_pattern_atlas.py')},
            'moseley.xrf_physics': { 'moseley.xrf_physics.ElementXRF': ( 'theoretical-peak-patterns.html#elementxrf',
                                                                         'moseley/xrf_physics.py'),
                                     'moseley.xrf_physics.ElementXRF.__init__': ( 'theoretical-peak-patterns.html#elementxrf.__init__',
                                                                                  'moseley/xrf_physics.py'),
                                     'moseley.xrf_physics.ElementXRF.plot_spectrum': ( 'theoretical-peak-patterns.html#elementxrf.plot_spectrum',
                                                                                       'moseley/xrf_physics.py'),
                                     'moseley.xrf_physics.find_peaks': ( 'theoretical-peak-patterns.html#find_peaks',
                                                                         'moseley/xrf_physics.py'),
                                     'moseley.xrf_physics.gaussian_convolve': ( 'theoretical-peak-patterns.html#gaussian_convolve',
                                                                                'moseley/xrf_physics.py'),
                                     'moseley.xrf_physics.get_attenuation': ( 'theoretical-peak-patterns.html#get_attenuation',
                                                                              'moseley/xrf_physics.py'),
                                     'moseley.xrf_physics.plot_pattern': ( 'theoretical-peak-patterns.html#plot_pattern',
                                                                           'moseley/xrf_physics.py')}}}
